/*
 * identify_libarc.c -- URL identify with libarc
 * (C)Copyright 2002 by Hiroshi Takekawa
 * This file is part of Enfle.
 *
 * Last Modified: Sun Oct 13 21:35:21 2002.
 * $Id$
 *
 * Enfle is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * Enfle is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#define REQUIRE_STRING_H
#define REQUIRE_UNISTD_H
#include "compat.h"
#include "common.h"

#include <libarc/arc.h>

#include "identify.h"

extern struct URL_module URL_module_file;
extern struct URL_module URL_module_http;
extern struct URL_module URL_module_ftp;
extern struct URL_module URL_module_news;

int
identify_url_libarc(Stream *st, char *path)
{
  URL u;
  int file_size = 0, get_num, mem_size = 0;
  char *region;
  char *read_pos;

  url_add_modules(
		  &URL_module_file,
		  &URL_module_http,
		  &URL_module_ftp,
		  &URL_module_news,
		  NULL);
  u = url_open(path);
  if (u == NULL) {
    debug_message_fnc("url_open(%s) failed\n", path);
    return IDENTIFY_FILE_SOPEN_FAILED;
  }

  mem_size = BUFSIZ * 2;
  region = read_pos = (char*)malloc(mem_size);
  while ((get_num = url_read(u, read_pos, BUFSIZ)) > 0) {
    file_size += get_num;
    read_pos += get_num;
    if ((mem_size - file_size) < BUFSIZ) {
      mem_size *= 2;
      region = (char*)realloc(region, mem_size);
      read_pos = region + file_size;
    }
  }
  url_close(u);

  debug_message_fnc("%d bytes read from %s\n", file_size, path);

  st->path = strdup(path);
  stream_make_memorystream(st, region, file_size);

  return IDENTIFY_FILE_STREAM;
}
